@@ -28,10 +28,12 @@ class TestFreeSpaceAccounting:
     
     def _get_free_space(self, reader: Reader) -> int:
         """Helper to get free space from OIEB"""
-        oieb = reader._read_oieb()
-        return oieb.payload_free_bytes
+        # Access the OIEB directly from reader
+        if reader._oieb:
+            return reader._oieb.payload_free_bytes
+        return 0
     
-    def test_free_space_after_simple_wrap(self):
+    def test_free_space_after_simple_wrap(self) -> None:
         """Test free space accounting after simple wrap"""
         buffer_name = f"test_freespace_wrap_{os.getpid()}"
         # Small buffer to force wrapping
@@ -55,6 +57,7 @@ class TestFreeSpaceAccounting:
                 # Read all frames to free up space
                 for i in range(8):
                     frame = reader.read_frame()
+                    assert frame is not None
                     assert len(frame.data) == frame_size
                     reader.release_frame(frame)
                 
@@ -66,13 +69,14 @@ class TestFreeSpaceAccounting:
                 
                 # Read the wrapped frame
                 frame = reader.read_frame()
+                assert frame is not None
                 assert len(frame.data) == frame_size
                 reader.release_frame(frame)
                 
                 final_free = self._get_free_space(reader)
                 assert final_free == initial_free
     
-    def test_free_space_after_multiple_wraps(self):
+    def test_free_space_after_multiple_wraps(self) -> None:
         """Test free space accounting after multiple wraps"""
         buffer_name = f"test_freespace_multi_{os.getpid()}"
         # Very small buffer to force frequent wraps
@@ -94,13 +98,14 @@ class TestFreeSpaceAccounting:
                     # Read all frames
                     for i in range(3):
                         frame = reader.read_frame()
+                        assert frame is not None
                         assert len(frame.data) == 120
                         reader.release_frame(frame)
                     
                     current_free = self._get_free_space(reader)
                     assert current_free == initial_free
     
-    def test_free_space_with_partial_reads(self):
+    def test_free_space_with_partial_reads(self) -> None:
         """Test free space with partial reads"""
         buffer_name = f"test_freespace_partial_{os.getpid()}"
         config = BufferConfig(metadata_size=100, payload_size=1000)
@@ -121,6 +126,7 @@ class TestFreeSpaceAccounting:
                 # Read only 2 frames
                 for i in range(2):
                     frame = reader.read_frame()
+                    assert frame is not None
                     reader.release_frame(frame)
                 
                 after_partial_free = self._get_free_space(reader)
@@ -130,12 +136,13 @@ class TestFreeSpaceAccounting:
                 # Read remaining 3 frames
                 for i in range(3):
                     frame = reader.read_frame()
+                    assert frame is not None
                     reader.release_frame(frame)
                 
                 final_free = self._get_free_space(reader)
                 assert final_free == initial_free
     
-    def test_wrap_marker_space_accounted_correctly(self):
+    def test_wrap_marker_space_accounted_correctly(self) -> None:
         """Test wrap marker space accounting"""
         buffer_name = f"test_wrap_marker_{os.getpid()}"
         config = BufferConfig(metadata_size=100, payload_size=1000)
@@ -149,6 +156,7 @@ class TestFreeSpaceAccounting:
                 for i in range(9):
                     writer.write_frame(data)
                     frame = reader.read_frame()
+                    assert frame is not None
                     reader.release_frame(frame)
                 
                 before_wrap_free = self._get_free_space(reader)
@@ -158,13 +166,14 @@ class TestFreeSpaceAccounting:
                 
                 # Read the frame (which includes processing wrap marker)
                 frame = reader.read_frame()
+                assert frame is not None
                 assert len(frame.data) == frame_size
                 reader.release_frame(frame)
                 
                 after_wrap_free = self._get_free_space(reader)
                 assert after_wrap_free == before_wrap_free
     
-    def test_stress_free_space_accounting(self):
+    def test_stress_free_space_accounting(self) -> None:
         """Stress test free space accounting"""
         buffer_name = f"test_stress_free_{os.getpid()}"
         config = BufferConfig(metadata_size=100, payload_size=10000)
@@ -213,7 +222,7 @@ class TestFreeSpaceAccounting:
 class TestAdvancedResourceCleanup:
     """Advanced resource cleanup tests"""
     
-    def test_create_destroy_multiple_times(self):
+    def test_create_destroy_multiple_times(self) -> None:
         """Test create/destroy multiple times"""
         buffer_name = f"test_multi_create_{os.getpid()}"
         config = BufferConfig(metadata_size=1024, payload_size=10240)
@@ -229,13 +238,14 @@ class TestAdvancedResourceCleanup:
                     writer.write_frame(data)
                     
                     frame = reader.read_frame()
+                    assert frame is not None
                     assert frame.data[0] == i
                     reader.release_frame(frame)
             
             # Small delay to ensure cleanup
             time.sleep(0.1)
     
-    def test_multiple_buffers_cleanup(self):
+    def test_multiple_buffers_cleanup(self) -> None:
         """Test multiple buffers cleanup"""
         buffer_names = [
             f"test_multi_buf_1_{os.getpid()}",
@@ -273,7 +283,7 @@ class TestAdvancedResourceCleanup:
                 if not reader._closed:
                     reader.close()
     
-    def test_cleanup_with_wrap_around(self):
+    def test_cleanup_with_wrap_around(self) -> None:
         """Test cleanup with buffer wrap-around"""
         buffer_name = f"test_cleanup_wrap_{os.getpid()}"
         small_buffer = 4096  # Small buffer to force wrap
@@ -288,6 +298,7 @@ class TestAdvancedResourceCleanup:
                     writer.write_frame(frame_data)
                     
                     frame = reader.read_frame()
+                    assert frame is not None
                     assert len(frame.data) == 512
                     assert frame.data[0] == i
                     reader.release_frame(frame)
@@ -298,6 +309,7 @@ class TestAdvancedResourceCleanup:
             with Writer(buffer_name) as writer:
                 writer.write_frame(b"\xFF")
                 frame = reader.read_frame()
+                assert frame is not None
                 assert frame.data[0] == 0xFF
                 reader.release_frame(frame)
 
@@ -305,7 +317,7 @@ class TestAdvancedResourceCleanup:
 class TestPatternValidation:
     """Pattern validation with hash tests"""
     
-    def test_pattern_validation_with_hash(self):
+    def test_pattern_validation_with_hash(self) -> None:
         """Test pattern validation with hash verification"""
         buffer_name = f"test_pattern_hash_{os.getpid()}"
         config = BufferConfig(metadata_size=1024, payload_size=256*1024)
@@ -338,21 +350,21 @@ class TestPatternValidation:
                     frame = reader.read_frame(timeout=5.0)
                     assert frame is not None, f"Frame {i} is None"
                     
-                    data = bytes(frame.data)
-                    assert len(data) == frame_size
+                    frame_data = bytes(frame.data)
+                    assert len(frame_data) == frame_size
                     
                     # Verify the pattern matches expected for this frame
                     expected_first = generate_pattern(i, 0)
                     expected_last = generate_pattern(i, frame_size - 1)
-                    assert data[0] == expected_first
-                    assert data[frame_size - 1] == expected_last
+                    assert frame_data[0] == expected_first
+                    assert frame_data[frame_size - 1] == expected_last
                     
                     # Sample check in the middle
                     expected_middle = generate_pattern(i, frame_size // 2)
-                    assert data[frame_size // 2] == expected_middle
+                    assert frame_data[frame_size // 2] == expected_middle
                     
                     # Verify hash
-                    actual_hash = hashlib.sha256(data).hexdigest()
+                    actual_hash = hashlib.sha256(frame_data).hexdigest()
                     assert actual_hash == frame_hashes[i]
                     
                     received_frames.add(i)
@@ -364,7 +376,7 @@ class TestPatternValidation:
 class TestRapidOperations:
     """Rapid operation tests"""
     
-    def test_rapid_write_read_cycles(self):
+    def test_rapid_write_read_cycles(self) -> None:
         """Test rapid write-read cycles"""
         buffer_name = f"test_rapid_cycles_{os.getpid()}"
         config = BufferConfig(metadata_size=1024, payload_size=64*1024)
@@ -396,7 +408,7 @@ class TestRapidOperations:
 class TestMetadataValidation:
     """Metadata validation tests"""
     
-    def test_metadata_write_once(self):
+    def test_metadata_write_once(self) -> None:
         """Test metadata write-once constraint"""
         buffer_name = f"test_metadata_once_{os.getpid()}"
         config = BufferConfig(metadata_size=1024, payload_size=64*1024)
@@ -413,9 +425,10 @@ class TestMetadataValidation:
                 
                 # Read metadata
                 read_metadata = reader.get_metadata()
+                assert read_metadata is not None
                 assert bytes(read_metadata) == metadata
     
-    def test_metadata_size_validation(self):
+    def test_metadata_size_validation(self) -> None:
         """Test metadata size validation"""
         buffer_name = f"test_metadata_size_{os.getpid()}"
         config = BufferConfig(metadata_size=256, payload_size=64*1024)
@@ -433,13 +446,14 @@ class TestMetadataValidation:
                 writer.set_metadata(small_metadata)
                 
                 read_metadata = reader.get_metadata()
+                assert read_metadata is not None
                 assert bytes(read_metadata) == small_metadata
 
 
 class TestSequentialOperations:
     """Sequential operation tests"""
     
-    def test_sequential_write_read(self):
+    def test_sequential_write_read(self) -> None:
         """Test sequential write/read within a single process"""
         buffer_name = f"test_sequential_{os.getpid()}"
         config = BufferConfig(metadata_size=1024, payload_size=64*1024)
@@ -462,7 +476,7 @@ class TestSequentialOperations:
                     assert value == i
                     reader.release_frame(frame)
     
-    def test_multiple_frames_sequential(self):
+    def test_multiple_frames_sequential(self) -> None:
         """Test multiple frames sequential"""
         buffer_name = f"test_multi_seq_{os.getpid()}"
         config = BufferConfig(metadata_size=1024, payload_size=256*1024)
