@@ -22,34 +22,44 @@ from zerobuffer.types import OIEB, FrameHeader, align_to_boundary
 class TestTypes:
     """Test core data structures"""
     
-    def test_oieb_pack_unpack(self):
-        """Test OIEB serialization"""
-        oieb = OIEB(
-            operation_size=128,
-            metadata_size=1024,
-            metadata_free_bytes=1024,
-            metadata_written_bytes=0,
-            payload_size=65536,
-            payload_free_bytes=65536,
-            payload_write_pos=0,
-            payload_read_pos=0,
-            payload_written_count=0,
-            payload_read_count=0,
-            writer_pid=1234,
-            reader_pid=5678
-        )
+    def test_oieb_view_direct_access(self) -> None:
+        """Test OIEBView direct memory access"""
+        from zerobuffer.oieb_view import OIEBView
         
-        # Pack and unpack
-        data = oieb.pack()
-        assert len(data) == 128  # OIEB is 128 bytes
+        # Create a bytearray to simulate shared memory
+        buffer = bytearray(128)
         
-        oieb2 = OIEB.unpack(data)
-        assert oieb2.operation_size == oieb.operation_size
-        assert oieb2.metadata_size == oieb.metadata_size
-        assert oieb2.writer_pid == oieb.writer_pid
-        assert oieb2.reader_pid == oieb.reader_pid
+        # Create view
+        oieb_view = OIEBView(memoryview(buffer))
+        
+        # Initialize with test values
+        oieb_view.oieb_size = 128
+        oieb_view.metadata_size = 1024
+        oieb_view.metadata_free_bytes = 1024
+        oieb_view.metadata_written_bytes = 0
+        oieb_view.payload_size = 65536
+        oieb_view.payload_free_bytes = 65536
+        oieb_view.payload_write_pos = 0
+        oieb_view.payload_read_pos = 0
+        oieb_view.payload_written_count = 0
+        oieb_view.payload_read_count = 0
+        oieb_view.writer_pid = 1234
+        oieb_view.reader_pid = 5678
+        
+        # Verify values are written directly to buffer
+        assert oieb_view.oieb_size == 128
+        assert oieb_view.metadata_size == 1024
+        assert oieb_view.writer_pid == 1234
+        assert oieb_view.reader_pid == 5678
+        
+        # Create another view on same buffer to verify data persistence
+        oieb_view2 = OIEBView(memoryview(buffer))
+        assert oieb_view2.oieb_size == 128
+        assert oieb_view2.metadata_size == 1024
+        assert oieb_view2.writer_pid == 1234
+        assert oieb_view2.reader_pid == 5678
     
-    def test_frame_header_pack_unpack(self):
+    def test_frame_header_pack_unpack(self) -> None:
         """Test FrameHeader serialization"""
         header = FrameHeader(payload_size=1024, sequence_number=42)
         
@@ -60,7 +70,7 @@ class TestTypes:
         assert header2.payload_size == header.payload_size
         assert header2.sequence_number == header.sequence_number
     
-    def test_align_to_boundary(self):
+    def test_align_to_boundary(self) -> None:
         """Test alignment function"""
         assert align_to_boundary(0) == 0
         assert align_to_boundary(1) == 64
@@ -73,11 +83,11 @@ class TestReaderWriter:
     """Test Reader and Writer basic functionality"""
     
     @pytest.fixture
-    def buffer_name(self):
+    def buffer_name(self) -> str:
         """Generate unique buffer name for each test"""
         return f"test_buffer_{os.getpid()}_{time.time()}"
     
-    def test_create_reader(self, buffer_name):
+    def test_create_reader(self, buffer_name: str) -> None:
         """Test creating a reader"""
         config = BufferConfig(metadata_size=1024, payload_size=64*1024)
         
@@ -87,7 +97,7 @@ class TestReaderWriter:
             assert reader.frames_read == 0
             assert reader.bytes_read == 0
     
-    def test_connect_writer(self, buffer_name):
+    def test_connect_writer(self, buffer_name: str) -> None:
         """Test connecting a writer to existing buffer"""
         config = BufferConfig(metadata_size=1024, payload_size=64*1024)
         
@@ -101,7 +111,7 @@ class TestReaderWriter:
                 assert writer.frames_written == 0
                 assert writer.bytes_written == 0
     
-    def test_multiple_writers_rejected(self, buffer_name):
+    def test_multiple_writers_rejected(self, buffer_name: str) -> None:
         """Test that only one writer can connect"""
         config = BufferConfig(metadata_size=1024, payload_size=64*1024)
         
@@ -111,12 +121,12 @@ class TestReaderWriter:
                 with pytest.raises(WriterAlreadyConnectedException):
                     Writer(buffer_name)
     
-    def test_writer_without_reader(self, buffer_name):
+    def test_writer_without_reader(self, buffer_name: str) -> None:
         """Test that writer cannot connect without reader"""
         with pytest.raises(ZeroBufferException):
             Writer(buffer_name)
     
-    def test_metadata_write_read(self, buffer_name):
+    def test_metadata_write_read(self, buffer_name: str) -> None:
         """Test metadata writing and reading"""
         config = BufferConfig(metadata_size=1024, payload_size=64*1024)
         
@@ -135,6 +145,8 @@ class TestReaderWriter:
                 read_metadata = reader.get_metadata()
                 assert read_metadata is not None
                 assert bytes(read_metadata) == metadata
+                # Release the memoryview reference before closing
+                del read_metadata
                 
                 # Cannot write metadata twice
                 with pytest.raises(MetadataAlreadyWrittenException):
@@ -149,11 +161,11 @@ class TestFrameOperations:
     """Test frame reading and writing"""
     
     @pytest.fixture
-    def buffer_name(self):
+    def buffer_name(self) -> str:
         """Generate unique buffer name for each test"""
         return f"test_buffer_{os.getpid()}_{time.time()}"
     
-    def test_single_frame_write_read(self, buffer_name):
+    def test_single_frame_write_read(self, buffer_name: str) -> None:
         """Test writing and reading a single frame"""
         config = BufferConfig(metadata_size=1024, payload_size=64*1024)
         
@@ -166,12 +178,10 @@ class TestFrameOperations:
                 # Read frame
                 frame = reader.read_frame(timeout=1.0)
                 assert frame is not None
-                assert frame.sequence == 1
-                assert frame.size == len(data)
-                assert bytes(frame.data) == data
-                
-                # Release frame
-                # Frame auto-released via context manager
+                with frame:  # Use context manager for proper cleanup
+                    assert frame.sequence == 1
+                    assert frame.size == len(data)
+                    assert bytes(frame.data) == data
                 
                 # Check stats
                 assert writer.frames_written == 1
@@ -179,7 +189,7 @@ class TestFrameOperations:
                 assert reader.frames_read == 1
                 assert reader.bytes_read == len(data)
     
-    def test_multiple_frames(self, buffer_name):
+    def test_multiple_frames(self, buffer_name: str) -> None:
         """Test writing and reading multiple frames"""
         config = BufferConfig(metadata_size=1024, payload_size=64*1024)
         
@@ -195,15 +205,15 @@ class TestFrameOperations:
                 for i in range(frames_to_write):
                     frame = reader.read_frame(timeout=1.0)
                     assert frame is not None
-                    assert frame.sequence == i + 1
-                    expected_data = f"Frame {i}".encode()
-                    assert bytes(frame.data) == expected_data
-                    # Frame auto-released via context manager
+                    with frame:  # Use context manager for proper cleanup
+                        assert frame.sequence == i + 1
+                        expected_data = f"Frame {i}".encode()
+                        assert bytes(frame.data) == expected_data
                 
                 assert writer.frames_written == frames_to_write
                 assert reader.frames_read == frames_to_write
     
-    def test_zero_copy_write(self, buffer_name):
+    def test_zero_copy_write(self, buffer_name: str) -> None:
         """Test zero-copy writing with memoryview"""
         config = BufferConfig(metadata_size=1024, payload_size=64*1024)
         
@@ -213,16 +223,16 @@ class TestFrameOperations:
                 data = bytearray(b"Zero-copy data")
                 view = memoryview(data)
                 
-                # Write with zero-copy
-                writer.write_frame_zero_copy(view)
+                # Write with zero-copy (memoryview is zero-copy)
+                writer.write_frame(view)
                 
                 # Read and verify
                 frame = reader.read_frame(timeout=1.0)
                 assert frame is not None
-                assert bytes(frame.data) == data
-                # Frame auto-released via context manager
+                with frame:  # Use context manager for proper cleanup
+                    assert bytes(frame.data) == data
     
-    def test_frame_too_large(self, buffer_name):
+    def test_frame_too_large(self, buffer_name: str) -> None:
         """Test writing frame larger than buffer"""
         config = BufferConfig(metadata_size=1024, payload_size=1024)  # Small buffer
         
@@ -233,51 +243,85 @@ class TestFrameOperations:
                 with pytest.raises(FrameTooLargeException):
                     writer.write_frame(large_data)
     
-    def test_buffer_wrap_around(self, buffer_name):
+    def test_buffer_wrap_around(self, buffer_name: str) -> None:
         """Test buffer wrap-around behavior"""
-        # Small buffer to force wrap-around
-        config = BufferConfig(metadata_size=64, payload_size=1024)
+        import threading
+        import queue
+        from typing import Tuple, Union
+        
+        # Use small buffer to test wrap-around
+        config = BufferConfig(metadata_size=256, payload_size=2048)
         
         with Reader(buffer_name, config) as reader:
             with Writer(buffer_name) as writer:
-                # Write frames that will cause wrap-around
-                frame_data = b"x" * 200  # Each frame ~216 bytes with header
+                frames_to_write = 20
+                frame_size = 256  # Small frames
+                results: queue.Queue[Tuple[str, Union[int, str]]] = queue.Queue()
                 
-                # Write first frame
-                writer.write_frame(frame_data)
+                def write_frames() -> None:
+                    """Writer thread"""
+                    try:
+                        for i in range(frames_to_write):
+                            data = bytes([i]) + b"x" * (frame_size - 1)
+                            writer.write_frame(data)
+                            results.put(("wrote", i))
+                    except Exception as e:
+                        results.put(("error", str(e)))
                 
-                # Read but don't release (buffer stays occupied)
-                frame1 = reader.read_frame(timeout=1.0)
-                assert frame1 is not None
+                def read_frames() -> None:
+                    """Reader thread"""
+                    try:
+                        for i in range(frames_to_write):
+                            frame = reader.read_frame(timeout=2.0)
+                            if frame:
+                                with frame:  # Use context manager for proper cleanup
+                                    assert frame.data[0] == i
+                                    assert len(frame.data) == frame_size
+                                    results.put(("read", i))
+                            else:
+                                results.put(("timeout", i))
+                                break
+                    except Exception as e:
+                        results.put(("error", str(e)))
                 
-                # Write more frames to fill buffer
-                writer.write_frame(frame_data)
-                writer.write_frame(frame_data)
-                writer.write_frame(frame_data)
+                # Start threads
+                writer_thread = threading.Thread(target=write_frames)
+                reader_thread = threading.Thread(target=read_frames)
                 
-                # Now release first frame to make space at beginning
-                # Frame auto-released via context manager
+                writer_thread.start()
+                reader_thread.start()
                 
-                # Write another frame (should wrap to beginning)
-                writer.write_frame(frame_data)
+                # Wait for completion
+                writer_thread.join(timeout=10)
+                reader_thread.join(timeout=10)
                 
-                # Read remaining frames
-                for i in range(4):
-                    frame = reader.read_frame(timeout=1.0)
-                    assert frame is not None
-                    assert bytes(frame.data) == frame_data
-                    # Frame auto-released via context manager
+                # Check results
+                writes = 0
+                reads = 0
+                errors = []
+                while not results.empty():
+                    event_type, value = results.get()
+                    if event_type == "wrote":
+                        writes += 1
+                    elif event_type == "read":
+                        reads += 1
+                    elif event_type == "error":
+                        errors.append(value)
+                
+                assert not errors, f"Errors occurred: {errors}"
+                assert writes == frames_to_write, f"Expected {frames_to_write} writes, got {writes}"
+                assert reads == frames_to_write, f"Expected {frames_to_write} reads, got {reads}"
 
 
 class TestErrorConditions:
     """Test error handling and edge cases"""
     
     @pytest.fixture
-    def buffer_name(self):
+    def buffer_name(self) -> str:
         """Generate unique buffer name for each test"""
         return f"test_buffer_{os.getpid()}_{time.time()}"
     
-    def test_reader_timeout(self, buffer_name):
+    def test_reader_timeout(self, buffer_name: str) -> None:
         """Test reader timeout when no data available"""
         config = BufferConfig(metadata_size=1024, payload_size=64*1024)
         
@@ -287,7 +331,7 @@ class TestErrorConditions:
                 frame = reader.read_frame(timeout=0.1)
                 assert frame is None  # Should timeout
     
-    def test_sequence_validation(self, buffer_name):
+    def test_sequence_validation(self, buffer_name: str) -> None:
         """Test sequence number validation"""
         config = BufferConfig(metadata_size=1024, payload_size=64*1024)
         
@@ -299,17 +343,19 @@ class TestErrorConditions:
                 
                 # Read first frame
                 frame1 = reader.read_frame(timeout=1.0)
-                assert frame1.sequence == 1
-                # Frame auto-released via context manager
+                assert frame1 is not None
+                with frame1:  # Use context manager for proper cleanup
+                    assert frame1.sequence == 1
                 
                 # Manually corrupt sequence in next frame
                 # This would require direct memory access - skip for now
                 # Just verify normal sequence works
                 frame2 = reader.read_frame(timeout=1.0)
-                assert frame2.sequence == 2
-                # Frame auto-released via context manager
+                assert frame2 is not None
+                with frame2:  # Use context manager for proper cleanup
+                    assert frame2.sequence == 2
     
-    def test_empty_frame_rejected(self, buffer_name):
+    def test_empty_frame_rejected(self, buffer_name: str) -> None:
         """Test that empty frames are rejected"""
         config = BufferConfig(metadata_size=1024, payload_size=64*1024)
         
@@ -323,34 +369,34 @@ class TestConcurrency:
     """Test concurrent operations"""
     
     @pytest.fixture
-    def buffer_name(self):
+    def buffer_name(self) -> str:
         """Generate unique buffer name for each test"""
         return f"test_buffer_{os.getpid()}_{time.time()}"
     
-    def test_concurrent_write_read(self, buffer_name):
+    def test_concurrent_write_read(self, buffer_name: str) -> None:
         """Test concurrent writing and reading"""
         config = BufferConfig(metadata_size=1024, payload_size=64*1024)
         frames_to_transfer = 100
         
-        def writer_thread(name):
+        def writer_thread(name: str) -> None:
             with Writer(name) as writer:
                 for i in range(frames_to_transfer):
                     data = f"Frame {i}".encode()
                     writer.write_frame(data)
                     time.sleep(0.001)  # Small delay
         
-        def reader_thread(name, results):
+        def reader_thread(name: str, results: list[list[int]]) -> None:
             with Reader(name, config) as reader:
                 frames_read = []
                 for i in range(frames_to_transfer):
                     frame = reader.read_frame(timeout=5.0)
                     if frame:
-                        frames_read.append(frame.sequence)
-                        # Frame auto-released via context manager
+                        with frame:  # Use context manager for proper cleanup
+                            frames_read.append(frame.sequence)
                 results.append(frames_read)
         
         # Start reader first
-        results = []
+        results: list[list[int]] = []
         reader_t = threading.Thread(target=reader_thread, args=(buffer_name, results))
         reader_t.start()
         
@@ -375,11 +421,11 @@ class TestZeroCopyVerification:
     """Verify zero-copy behavior"""
     
     @pytest.fixture
-    def buffer_name(self):
+    def buffer_name(self) -> str:
         """Generate unique buffer name for each test"""
         return f"test_buffer_{os.getpid()}_{time.time()}"
     
-    def test_memoryview_zero_copy(self, buffer_name):
+    def test_memoryview_zero_copy(self, buffer_name: str) -> None:
         """Test that memoryview provides zero-copy access"""
         config = BufferConfig(metadata_size=1024, payload_size=64*1024)
         
@@ -393,21 +439,24 @@ class TestZeroCopyVerification:
                 frame = reader.read_frame(timeout=1.0)
                 assert frame is not None
                 
-                # Get memoryview of frame data
-                frame_view = frame.data
-                assert isinstance(frame_view, memoryview)
-                
-                # Verify data matches without copying
-                assert frame_view.tobytes() == original_data
-                
-                # Test that we can slice without copying
-                slice_view = frame_view[0:8]
-                assert isinstance(slice_view, memoryview)
-                assert slice_view.tobytes() == original_data[0:8]
-                
-                # Frame auto-released via context manager
+                with frame:  # Use context manager for proper cleanup
+                    # Get memoryview of frame data
+                    frame_view = frame.data
+                    assert isinstance(frame_view, memoryview)
+                    
+                    # Verify data matches without copying
+                    assert frame_view.tobytes() == original_data
+                    
+                    # Test that we can slice without copying
+                    slice_view = frame_view[0:8]
+                    assert isinstance(slice_view, memoryview)
+                    assert slice_view.tobytes() == original_data[0:8]
+                    
+                    # Clean up local references
+                    del slice_view
+                    del frame_view
     
-    def test_direct_buffer_access(self, buffer_name):
+    def test_direct_buffer_access(self, buffer_name: str) -> None:
         """Test direct buffer access API"""
         config = BufferConfig(metadata_size=1024, payload_size=64*1024)
         
@@ -423,11 +472,14 @@ class TestZeroCopyVerification:
                 # Commit the frame
                 writer.commit_frame()
                 
+                # Release the memoryview reference
+                del buffer
+                
                 # Read and verify
                 frame = reader.read_frame(timeout=1.0)
                 assert frame is not None
-                assert bytes(frame.data) == data_to_write
-                # Frame auto-released via context manager
+                with frame:  # Use context manager for proper cleanup
+                    assert bytes(frame.data) == data_to_write
 
 
 if __name__ == "__main__":
