@@ -7,6 +7,6 @@ between C++, C#, and Python implementations.
 
 from .writer import main as writer_main
 from .reader import main as reader_main
-from .relay import main as relay_main
+# relay_main not yet implemented
 
-__all__ = ['writer_main', 'reader_main', 'relay_main']
\ No newline at end of file
+__all__ = ['writer_main', 'reader_main']
\ No newline at end of file
