@@ -6,11 +6,13 @@ Provides zero-copy writing to shared memory buffers.
 
 import os
 import threading
-from typing import Optional, Union
+from datetime import timedelta
+from typing import Optional, Union, Any
 import logging
 
 from . import platform
-from .types import OIEB, Frame, FrameHeader, align_to_boundary
+from .types import Frame, FrameHeader, align_to_boundary
+from .oieb_view import OIEBView
 from .exceptions import (
     ZeroBufferException,
     ReaderDeadException,
@@ -51,6 +53,8 @@ class Writer(LoggerMixin):
         self._frames_written = 0
         self._bytes_written = 0
         self._metadata_written = False
+        self._write_timeout = timedelta(seconds=5)  # Default timeout
+        self._oieb: Optional[OIEBView] = None  # Will be initialized after shm is opened
         
         # Set logger if provided, otherwise use mixin
         if logger:
@@ -63,57 +67,52 @@ class Writer(LoggerMixin):
             # Open existing shared memory using the new abstraction
             self._shm = SharedMemoryFactory.open(name)
             
-            # Read OIEB to get layout using the clean API
-            oieb_data = self._shm.read_bytes(0, OIEB.SIZE)
-            oieb = OIEB.unpack(oieb_data)
+            # Initialize OIEB view for direct memory access
+            self._oieb = OIEBView(self._shm.get_memoryview(0, OIEBView.SIZE))
             
             # Verify OIEB
-            if oieb.oieb_size != 128:
-                raise ZeroBufferException(f"Invalid OIEB size: {oieb.oieb_size} - version mismatch?")
+            if self._oieb.oieb_size != 128:
+                raise ZeroBufferException(f"Invalid OIEB size: {self._oieb.oieb_size} - version mismatch?")
             
             # Check if reader exists
-            if oieb.reader_pid == 0 or not platform.process_exists(oieb.reader_pid):
+            if self._oieb.reader_pid == 0 or not platform.process_exists(self._oieb.reader_pid):
                 raise ZeroBufferException("No active reader found")
             
             # Check if another writer exists
-            if oieb.writer_pid != 0 and platform.process_exists(oieb.writer_pid):
+            if self._oieb.writer_pid != 0 and platform.process_exists(self._oieb.writer_pid):
                 raise WriterAlreadyConnectedException()
             
-            # Set writer PID
-            oieb.writer_pid = os.getpid()
-            self._logger.debug("Setting writer PID=%d in OIEB", oieb.writer_pid)
-            self._logger.debug("OIEB before pack: writer_pid=%d, reader_pid=%d", oieb.writer_pid, oieb.reader_pid)
-            
-            # Write OIEB back using the clean API
-            packed_data = oieb.pack()
-            self._logger.debug("Writing OIEB with writer_pid=%d", oieb.writer_pid)
-            self._shm.write_bytes(0, packed_data)
-            
-            # Verify it was written using the clean API
-            verify_data = self._shm.read_bytes(0, OIEB.SIZE)
-            verify_oieb = OIEB.unpack(verify_data)
-            self._logger.debug("Verified writer PID after write: %d", verify_oieb.writer_pid)
-            
-            self._logger.debug("Writer PID set successfully")
+            # Set writer PID directly in shared memory (OIEBView updates immediately)
+            self._oieb.writer_pid = os.getpid()
+            self._logger.debug("Set writer PID=%d directly in OIEB shared memory", self._oieb.writer_pid)
+            self._logger.debug("OIEB state: writer_pid=%d, reader_pid=%d", self._oieb.writer_pid, self._oieb.reader_pid)
             
             # Store layout info
-            self._oieb_size = align_to_boundary(oieb.oieb_size)
-            self._metadata_size = oieb.metadata_size
-            self._payload_size = oieb.payload_size
+            self._oieb_size = align_to_boundary(self._oieb.oieb_size)
+            self._metadata_size = self._oieb.metadata_size
+            self._payload_size = self._oieb.payload_size
             
             # Open semaphores
             self._sem_write = platform.open_semaphore(f"sem-w-{name}")
             self._sem_read = platform.open_semaphore(f"sem-r-{name}")
             
             # Check if metadata was already written
-            self._metadata_written = (oieb.metadata_written_bytes > 0)
+            self._metadata_written = (self._oieb.metadata_written_bytes > 0)
             
+        except FileNotFoundError:
+            # No shared memory found - reader must be created first
+            self._cleanup_on_error()
+            raise ZeroBufferException(f"No shared memory found for buffer '{name}' - reader must be created first")
         except Exception:
             self._cleanup_on_error()
             raise
     
     def _cleanup_on_error(self) -> None:
         """Clean up resources on initialization error"""
+        # Dispose OIEBView first to release memoryview
+        if hasattr(self, '_oieb') and self._oieb:
+            self._oieb.dispose()
+            self._oieb = None
         if hasattr(self, '_sem_read'):
             self._sem_read.close()
         if hasattr(self, '_sem_write'):
@@ -121,16 +120,8 @@ class Writer(LoggerMixin):
         if hasattr(self, '_shm'):
             self._shm.close()
     
-    def _read_oieb(self) -> OIEB:
-        """Read current OIEB from shared memory"""
-        # Read OIEB using the clean API
-        oieb_data = self._shm.read_bytes(0, OIEB.SIZE)
-        return OIEB.unpack(oieb_data)
-    
-    def _write_oieb(self, oieb: OIEB) -> None:
-        """Write OIEB to shared memory"""
-        # Write OIEB using the clean API
-        self._shm.write_bytes(0, oieb.pack())
+    # OIEB access is now direct through self._oieb (OIEBView)
+    # No need for _read_oieb or _write_oieb methods
     
     def set_metadata(self, data: Union[bytes, bytearray, memoryview]) -> None:
         """
@@ -150,11 +141,12 @@ class Writer(LoggerMixin):
             if self._metadata_written:
                 raise MetadataAlreadyWrittenException()
             
-            oieb = self._read_oieb()
+            if not self._oieb:
+                raise ZeroBufferException("Writer not properly initialized")
             
             # Check size with header
             total_size = 8 + len(data)  # 8 bytes for size prefix
-            if total_size > oieb.metadata_size:
+            if total_size > self._oieb.metadata_size:
                 raise ZeroBufferException("Metadata too large for buffer")
             
             # Write size prefix
@@ -172,21 +164,19 @@ class Writer(LoggerMixin):
                     self._shm.write_bytes(metadata_offset + 8, data)
             
             # Update OIEB
-            oieb.metadata_written_bytes = total_size
-            oieb.metadata_free_bytes = oieb.metadata_size - total_size
+            self._oieb.metadata_written_bytes = total_size
+            self._oieb.metadata_free_bytes = self._oieb.metadata_size - total_size
             
             self._logger.info("Updating OIEB after metadata write:")
-            self._logger.info("  metadata_written_bytes: %d", oieb.metadata_written_bytes)
-            self._logger.info("  metadata_free_bytes: %d", oieb.metadata_free_bytes)
-            self._logger.info("  payload_free_bytes (unchanged): %d", oieb.payload_free_bytes)
+            self._logger.info("  metadata_written_bytes: %d", self._oieb.metadata_written_bytes)
+            self._logger.info("  metadata_free_bytes: %d", self._oieb.metadata_free_bytes)
+            self._logger.info("  payload_free_bytes (unchanged): %d", self._oieb.payload_free_bytes)
             
-            self._write_oieb(oieb)
             
-            # Verify the write
-            oieb_verify = self._read_oieb()
+            # Verify the write directly from shared memory
             self._logger.info("Verified OIEB after metadata write:")
-            self._logger.info("  metadata_written_bytes: %d", oieb_verify.metadata_written_bytes)
-            self._logger.info("  payload_free_bytes: %d", oieb_verify.payload_free_bytes)
+            self._logger.info("  metadata_written_bytes: %d", self._oieb.metadata_written_bytes)
+            self._logger.info("  payload_free_bytes: %d", self._oieb.payload_free_bytes)
             
             self._metadata_written = True
     
@@ -197,19 +187,21 @@ class Writer(LoggerMixin):
         else:
             return buffer_size - read_pos + write_pos
     
-    def _get_continuous_free_space(self, oieb: OIEB) -> int:
+    def _get_continuous_free_space(self) -> int:
         """Calculate continuous free space in buffer"""
-        if oieb.payload_write_pos >= oieb.payload_read_pos:
+        if not self._oieb:
+            return 0
+        if self._oieb.payload_write_pos >= self._oieb.payload_read_pos:
             # Write ahead of read - check space to end and beginning
-            space_to_end = oieb.payload_size - oieb.payload_write_pos
-            if oieb.payload_read_pos == 0:
+            space_to_end = self._oieb.payload_size - self._oieb.payload_write_pos
+            if self._oieb.payload_read_pos == 0:
                 # Can't wrap if reader at beginning
                 return space_to_end
             # Can use space at beginning if we wrap
-            return max(space_to_end, oieb.payload_read_pos)
+            return max(space_to_end, self._oieb.payload_read_pos)
         else:
             # Read ahead of write - continuous space until read pos
-            return oieb.payload_read_pos - oieb.payload_write_pos
+            return self._oieb.payload_read_pos - self._oieb.payload_write_pos
     
     
     def write_frame(self, data: Union[bytes, bytearray, memoryview]) -> None:
@@ -217,7 +209,7 @@ class Writer(LoggerMixin):
         Write a frame to the buffer
         
         This method copies the data into the shared memory buffer.
-        For true zero-copy writing with memoryview, use write_frame_zero_copy.
+        When passed a memoryview, this is a true zero-copy operation.
         
         Args:
             data: Frame data to write
@@ -236,28 +228,30 @@ class Writer(LoggerMixin):
             if self._closed:
                 raise ZeroBufferException("Writer is closed")
             
+            if not self._oieb:
+                raise ZeroBufferException("Writer not properly initialized")
+            
             frame_size = len(data)
             total_size = FrameHeader.SIZE + frame_size
             
             # Early check if reader has disconnected gracefully
-            oieb = self._read_oieb()
-            if oieb.reader_pid == 0:
+            if self._oieb.reader_pid == 0:
                 raise ReaderDeadException()
             
             # Check if frame is too large
-            if total_size > oieb.payload_size:
+            if total_size > self._oieb.payload_size:
                 raise FrameTooLargeException()
             
             while True:
                 # Check if reader hasn't exited gracefully
-                if oieb.reader_pid == 0:
+                if self._oieb.reader_pid == 0:
                     raise ReaderDeadException()
                 
                 self._logger.debug("Write frame check: total_size=%d, payload_free_bytes=%d, payload_size=%d", 
-                                 total_size, oieb.payload_free_bytes, oieb.payload_size)
+                                 total_size, self._oieb.payload_free_bytes, self._oieb.payload_size)
                 
                 # Check for available space
-                if oieb.payload_free_bytes >= total_size:
+                if self._oieb.payload_free_bytes >= total_size:
                     # We have enough space
                     self._logger.debug("Have enough space, breaking from wait loop")
                     break
@@ -265,53 +259,49 @@ class Writer(LoggerMixin):
                 # Log detailed state before waiting
                 self._logger.info("About to wait on semaphore - not enough space")
                 self._logger.info("  total_size needed: %d", total_size)
-                self._logger.info("  payload_free_bytes: %d", oieb.payload_free_bytes)
-                self._logger.info("  payload_size: %d", oieb.payload_size)
-                self._logger.info("  payload_write_pos: %d", oieb.payload_write_pos)
-                self._logger.info("  payload_read_pos: %d", oieb.payload_read_pos)
-                self._logger.info("  metadata_written_bytes: %d", oieb.metadata_written_bytes)
-                self._logger.info("  reader_pid: %d", oieb.reader_pid)
-                self._logger.info("  writer_pid: %d", oieb.writer_pid)
+                self._logger.info("  payload_free_bytes: %d", self._oieb.payload_free_bytes)
+                self._logger.info("  payload_size: %d", self._oieb.payload_size)
+                self._logger.info("  payload_write_pos: %d", self._oieb.payload_write_pos)
+                self._logger.info("  payload_read_pos: %d", self._oieb.payload_read_pos)
+                self._logger.info("  metadata_written_bytes: %d", self._oieb.metadata_written_bytes)
+                self._logger.info("  reader_pid: %d", self._oieb.reader_pid)
+                self._logger.info("  writer_pid: %d", self._oieb.writer_pid)
                 
-                # Re-read OIEB just to check if it changed
-                oieb_recheck = self._read_oieb()
-                if oieb_recheck.payload_free_bytes != oieb.payload_free_bytes:
-                    self._logger.warning("OIEB changed during check! free_bytes was %d, now %d", 
-                                       oieb.payload_free_bytes, oieb_recheck.payload_free_bytes)
+                # Log current OIEB state (direct from shared memory)
+                self._logger.debug("Current OIEB state: free_bytes=%d", self._oieb.payload_free_bytes)
                 
                 # Wait for reader to free space (blocking)
                 self._logger.info("Waiting on sem_read semaphore...")
-                if not self._sem_read.acquire(timeout=5.0):
+                if not self._sem_read.acquire(timeout=self._write_timeout.total_seconds()):
                     # Timeout - check if reader is alive
-                    if oieb.reader_pid == 0 or not platform.process_exists(oieb.reader_pid):
+                    if self._oieb.reader_pid == 0 or not platform.process_exists(self._oieb.reader_pid):
                         raise ReaderDeadException()
                     # Buffer is full - throw exception like C# does
                     raise BufferFullException()
                 
                 # Re-read OIEB after semaphore for next iteration
-                oieb = self._read_oieb()
-            
+                
             # Check if we need to wrap
-            continuous_free = self._get_continuous_free_space(oieb)
-            space_to_end = oieb.payload_size - oieb.payload_write_pos
+            continuous_free = self._get_continuous_free_space()
+            space_to_end = self._oieb.payload_size - self._oieb.payload_write_pos
             
             # We need to wrap if frame doesn't fit in continuous space
-            if continuous_free >= total_size and space_to_end < total_size and oieb.payload_read_pos > 0:
+            if continuous_free >= total_size and space_to_end < total_size and self._oieb.payload_read_pos > 0:
                 # Need to wrap to beginning
                 # Write a special marker if there's space for at least a header
                 if space_to_end >= FrameHeader.SIZE:
                     # Write wrap marker header
                     wrap_header = FrameHeader(payload_size=0, sequence_number=0)
                     payload_base = self._oieb_size + self._metadata_size
-                    wrap_offset = payload_base + oieb.payload_write_pos
+                    wrap_offset = payload_base + self._oieb.payload_write_pos
                     self._shm.write_bytes(wrap_offset, wrap_header.pack())
                 
                 # Account for the wasted space at the end
-                oieb.payload_free_bytes -= space_to_end
+                self._oieb.payload_free_bytes -= space_to_end
                 
                 # Move to beginning of buffer
-                oieb.payload_write_pos = 0
-                oieb.payload_written_count += 1  # Count the wrap marker
+                self._oieb.payload_write_pos = 0
+                self._oieb.payload_written_count += 1  # Count the wrap marker
             
             # Write frame header
             header = FrameHeader(
@@ -319,53 +309,29 @@ class Writer(LoggerMixin):
                 sequence_number=self._sequence_number
             )
             payload_base = self._oieb_size + self._metadata_size
-            header_offset = payload_base + oieb.payload_write_pos
+            header_offset = payload_base + self._oieb.payload_write_pos
             self._shm.write_bytes(header_offset, header.pack())
             
             # Write frame data
             data_offset = header_offset + FrameHeader.SIZE
-            self._shm.write_bytes(data_offset, data)
+            self._shm.write_bytes(data_offset, bytes(data))
             
             # Update tracking
-            oieb.payload_write_pos += total_size
+            self._oieb.payload_write_pos += total_size
             self._sequence_number += 1
             self._frames_written += 1
             self._bytes_written += frame_size
             
             # Update OIEB
-            oieb.payload_free_bytes -= total_size
-            oieb.payload_written_count += 1
+            self._oieb.payload_free_bytes -= total_size
+            self._oieb.payload_written_count += 1
             
             # Memory barrier equivalent (handled by lock)
             
-            self._write_oieb(oieb)
             
             # Signal reader
             self._sem_write.release()
     
-    def write_frame_zero_copy(self, data: memoryview) -> None:
-        """
-        Write a frame with true zero-copy operation
-        
-        This method requires the data to be a memoryview and directly copies
-        from the source buffer to shared memory without intermediate copies.
-        
-        Args:
-            data: Frame data as memoryview
-            
-        Raises:
-            TypeError: If data is not a memoryview
-            InvalidFrameSizeException: If data is empty
-            FrameTooLargeException: If frame is too large for buffer
-            ReaderDeadException: If reader process died
-        """
-        if not isinstance(data, memoryview):
-            raise TypeError("write_frame_zero_copy requires memoryview for zero-copy operation")
-        
-        # Use the same implementation as write_frame since memoryview assignment
-        # in Python is already zero-copy when possible
-        self.write_frame(data)
-    
     def get_frame_buffer(self, size: int) -> memoryview:
         """
         Get a buffer for direct writing (advanced zero-copy API)
@@ -394,54 +360,56 @@ class Writer(LoggerMixin):
                 self._lock.release()
                 raise ZeroBufferException("Writer is closed")
             
+            if not self._oieb:
+                self._lock.release()
+                raise ZeroBufferException("Writer not properly initialized")
+            
             total_size = FrameHeader.SIZE + size
             
             # Early check if reader has disconnected gracefully
-            oieb = self._read_oieb()
-            if oieb.reader_pid == 0:
+            if self._oieb.reader_pid == 0:
                 raise ReaderDeadException()
             
-            if total_size > oieb.payload_size:
+            if total_size > self._oieb.payload_size:
                 raise FrameTooLargeException()
             
             # Wait for space (same logic as write_frame)
             while True:
                 # Check if reader hasn't exited gracefully
-                if oieb.reader_pid == 0:
+                if self._oieb.reader_pid == 0:
                     raise ReaderDeadException()
                 
                 # Check for available space
-                if oieb.payload_free_bytes >= total_size:
+                if self._oieb.payload_free_bytes >= total_size:
                     break
                 
-                if not self._sem_read.acquire(timeout=5.0):
-                    if oieb.reader_pid == 0 or not platform.process_exists(oieb.reader_pid):
+                if not self._sem_read.acquire(timeout=self._write_timeout.total_seconds()):
+                    if self._oieb.reader_pid == 0 or not platform.process_exists(self._oieb.reader_pid):
                         raise ReaderDeadException()
                     # Buffer is full - throw exception like C# does
                     raise BufferFullException()
                 
                 # Re-read OIEB after semaphore for next iteration
-                oieb = self._read_oieb()
-            
+                
             # Handle wrap-around if needed
-            continuous_free = self._get_continuous_free_space(oieb)
-            space_to_end = oieb.payload_size - oieb.payload_write_pos
+            continuous_free = self._get_continuous_free_space()
+            space_to_end = self._oieb.payload_size - self._oieb.payload_write_pos
             
             # We need to wrap if frame doesn't fit in continuous space
-            if continuous_free >= total_size and space_to_end < total_size and oieb.payload_read_pos > 0:
+            if continuous_free >= total_size and space_to_end < total_size and self._oieb.payload_read_pos > 0:
                 if space_to_end >= FrameHeader.SIZE:
                     # Write wrap marker header
                     wrap_header = FrameHeader(payload_size=0, sequence_number=0)
                     payload_base = self._oieb_size + self._metadata_size
-                    wrap_offset = payload_base + oieb.payload_write_pos
+                    wrap_offset = payload_base + self._oieb.payload_write_pos
                     self._shm.write_bytes(wrap_offset, wrap_header.pack())
                 
                 # Account for the wasted space at the end
-                oieb.payload_free_bytes -= space_to_end
+                self._oieb.payload_free_bytes -= space_to_end
                 
                 # Move to beginning of buffer
-                oieb.payload_write_pos = 0
-                oieb.payload_written_count += 1  # Count the wrap marker
+                self._oieb.payload_write_pos = 0
+                self._oieb.payload_written_count += 1  # Count the wrap marker
             
             # Write frame header
             header = FrameHeader(
@@ -449,11 +417,11 @@ class Writer(LoggerMixin):
                 sequence_number=self._sequence_number
             )
             payload_base = self._oieb_size + self._metadata_size
-            header_offset = payload_base + oieb.payload_write_pos
+            header_offset = payload_base + self._oieb.payload_write_pos
             self._shm.write_bytes(header_offset, header.pack())
             
             # Store state for commit
-            self._pending_write_pos = oieb.payload_write_pos + FrameHeader.SIZE
+            self._pending_write_pos = self._oieb.payload_write_pos + FrameHeader.SIZE
             self._pending_frame_size = size
             self._pending_total_size = total_size
             
@@ -476,19 +444,19 @@ class Writer(LoggerMixin):
             raise ZeroBufferException("No pending frame to commit")
         
         try:
-            oieb = self._read_oieb()
+            if not self._oieb:
+                raise ZeroBufferException("Writer not properly initialized")
             
             # Update write position
-            oieb.payload_write_pos = (self._pending_write_pos + self._pending_frame_size) % oieb.payload_size
+            self._oieb.payload_write_pos = (self._pending_write_pos + self._pending_frame_size) % self._oieb.payload_size
             self._sequence_number += 1
             self._frames_written += 1
             self._bytes_written += self._pending_frame_size
             
             # Update OIEB
-            oieb.payload_free_bytes -= self._pending_total_size
-            oieb.payload_written_count += 1
+            self._oieb.payload_free_bytes -= self._pending_total_size
+            self._oieb.payload_written_count += 1
             
-            self._write_oieb(oieb)
             
             # Signal reader
             self._sem_write.release()
@@ -502,11 +470,11 @@ class Writer(LoggerMixin):
             # Must unlock here since we locked in get_frame_buffer
             self._lock.release()
     
-    def _is_reader_connected(self, oieb: Optional[OIEB] = None) -> bool:
+    def _is_reader_connected(self) -> bool:
         """Check if reader is connected"""
-        if oieb is None:
-            oieb = self._read_oieb()
-        return oieb.reader_pid != 0 and platform.process_exists(oieb.reader_pid)
+        if not self._oieb:
+            return False
+        return self._oieb.reader_pid != 0 and platform.process_exists(self._oieb.reader_pid)
     
     def is_reader_connected(self) -> bool:
         """Check if reader is connected"""
@@ -525,6 +493,31 @@ class Writer(LoggerMixin):
         """Get number of bytes written"""
         return self._bytes_written
     
+    @property
+    def write_timeout(self) -> timedelta:
+        """
+        Get the timeout for write operations when the buffer is full.
+        
+        Returns:
+            Timeout as timedelta (default is 5 seconds)
+        """
+        return self._write_timeout
+    
+    @write_timeout.setter
+    def write_timeout(self, value: timedelta) -> None:
+        """
+        Set the timeout for write operations when the buffer is full.
+        
+        Args:
+            value: Timeout as timedelta (must be positive)
+            
+        Raises:
+            ValueError: If timeout is not positive
+        """
+        if value <= timedelta():
+            raise ValueError("Write timeout must be positive")
+        self._write_timeout = value
+    
     def close(self) -> None:
         """Close the writer and clean up resources"""
         with self._lock:
@@ -535,13 +528,15 @@ class Writer(LoggerMixin):
             
             # Clear writer PID
             try:
-                oieb = self._read_oieb()
-                oieb.writer_pid = 0
-                self._write_oieb(oieb)
+                if self._oieb:
+                    self._oieb.writer_pid = 0
             except:
                 pass
             
-            # No persistent memoryviews to release anymore
+            # Properly dispose the OIEBView to release its memoryview
+            if self._oieb:
+                self._oieb.dispose()
+                self._oieb = None
             
             # Close resources (writer doesn't own them)
             if hasattr(self, '_sem_read'):
@@ -553,11 +548,11 @@ class Writer(LoggerMixin):
             if hasattr(self, '_shm'):
                 self._shm.close()
     
-    def __enter__(self):
+    def __enter__(self) -> 'Writer':
         return self
     
-    def __exit__(self, exc_type, exc_val, exc_tb):
+    def __exit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> None:
         self.close()
     
-    def __del__(self):
+    def __del__(self) -> None:
         self.close()
\ No newline at end of file
