@@ -27,7 +27,7 @@ class BasicCommunicationSteps(BaseSteps):
         self._writers: Dict[str, Writer] = {}
         self._last_frame: Optional[Union[Frame, Dict[str, Any]]] = None
         self._frames_written: List[Dict[str, Any]] = []
-        self._frames_read: List[Frame] = []
+        self._frames_read: List[Dict[str, Any]] = []
         self._write_error: Optional[Exception] = None
         self._buffer_naming = BufferNamingService(self.logger)
         self._current_buffer = ""
@@ -72,9 +72,6 @@ class BasicCommunicationSteps(BaseSteps):
             f"payload_size={payload_size}"
         )
         
-        # Small delay to ensure shared memory is fully initialized
-        await asyncio.sleep(0.1)
-        
     @when(r"(?:the '([^']+)' process )?connects to buffer '([^']+)'")
     async def connect_to_buffer(self, process: Optional[str], buffer_name: str) -> None:
         """Connect a writer to an existing buffer"""
@@ -88,9 +85,6 @@ class BasicCommunicationSteps(BaseSteps):
         
         self.logger.info(f"Connected to buffer '{buffer_name}' (actual: '{actual_buffer_name}'")
         
-        # Small delay to ensure shared memory updates are visible
-        await asyncio.sleep(0.1)
-        
     @when(r"(?:the '([^']+)' process )?writes metadata with size '(\d+)'")
     async def write_metadata(self, process: Optional[str], size: str) -> None:
         """Write metadata to the buffer"""
@@ -111,9 +105,6 @@ class BasicCommunicationSteps(BaseSteps):
         
         self.logger.info(f"Wrote metadata with size {size}")
         
-        # Small delay to ensure OIEB updates are visible
-        await asyncio.sleep(0.1)
-        
     @when(r"(?:the '([^']+)' process )?writes frame with size '(\d+)' and sequence '(\d+)'")
     async def write_frame_with_sequence(self, process: Optional[str], size: str, sequence: str) -> None:
         """Write a frame with specific size and sequence"""
@@ -326,8 +317,9 @@ class BasicCommunicationSteps(BaseSteps):
         else:
             raise TypeError(f"Unexpected type for _last_frame: {type(self._last_frame)}")
         
+        assert frame_sequence is not None, "Frame sequence should not be None"
         # Generate expected data using the shared pattern
-        expected_data = TestDataPatterns.generate_frame_data(len(frame_data), frame_sequence)
+        expected_data = TestDataPatterns.generate_frame_data(len(frame_data), int(frame_sequence))
         
         # Compare frame data with expected data
         assert frame_data == expected_data, "Frame data does not match expected pattern"
@@ -482,7 +474,8 @@ class BasicCommunicationSteps(BaseSteps):
             raise TypeError(f"Unexpected type for _last_frame: {type(self._last_frame)}")
         
         # Generate the expected test pattern based on the frame's sequence number
-        expected_pattern = TestDataPatterns.generate_frame_data(len(frame_data), sequence)
+        assert sequence is not None, "Frame sequence should not be None"
+        expected_pattern = TestDataPatterns.generate_frame_data(len(frame_data), int(sequence))
         
         assert frame_data == expected_pattern, "Frame data does not match test pattern"
         
@@ -511,7 +504,13 @@ class BasicCommunicationSteps(BaseSteps):
                 assert TestDataPatterns.verify_simple_frame_data(frame_data), \
                     f"Frame {i+1} data does not match expected pattern"
                     
-            self._frames_read.append(frame)
+                # Store frame info as dict - MUST be done inside the context manager
+                frame_info = {
+                    'sequence': frame.sequence,
+                    'size': len(frame.data),
+                    'data': bytes(frame.data)
+                }
+                self._frames_read.append(frame_info)
             
         self.logger.info(f"Read {count} frames with correct sizes")
         
